/*
Teeny Win32 demo program.

Orion Sky Lawlor, olawlor@acm.org, 2006/04/18 (Public Domain)
*/
#undef UNICODE /* Use ASCII versions of Windows calls, not Unicode */
#define _CRT_SECURE_NO_DEPRECATE 1 /* don't whine about fopen, man... */

#include <windows.h> //<- main windows header
#include <resources/resource.h> // <- windows resource IDs
#include <cstdio>
#include <string.h>
#include <iostream>
#include <sstream>
#include <string>
#include <vector>

/* Windows libraries.  Should be listed on the command line, but just in case... */
#pragma comment (lib, "kernel32.lib") 
#pragma comment (lib, "user32.lib") 
#pragma comment (lib, "gdi32.lib") 


/* Function Prototypes */
LRESULT CALLBACK myWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK myAboutBox(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
void windowRepaint(void);
void handleMouse(int x,int y,int keys,UINT msg);



/************** Globals **********/
const char *WindowClass_name="OSL_GUI_Wndproc1_0";
HINSTANCE hInst; // current application instance
HWND hWnd; //Main window handle
HWND hEdit1; //Edit window 1's handle
HWND hBut1; // Button

/************** Printouts to a circular buffer ***********/
enum {nBuf=10}; int curBuf=0;
std::ostringstream printBuf[nBuf]; /* circular list of prints */
std::ostringstream &print(void) {
	std::ostringstream &ret=printBuf[(curBuf++)%nBuf];
	ret.str(""); /* clear old contents of this slot */
	InvalidateRect(hWnd,NULL,false); /* redraw window when you get a chance */
	return ret;
}

/********************** Actual Code Starts Here **************************/

/* Main program */
int APIENTRY WinMain(HINSTANCE hInst_,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine, /* command-line args */
                     int       nCmdShow)
{
	hInst=hInst_;
	print()<<"Started up.  Instance is "<<hInst<<"\n";

	//Register our new "window class" so we can handle events.
	WNDCLASSEX wcex;
	memset(&wcex,0,sizeof(wcex)); /* Set everything unused to zero */
	wcex.cbSize = sizeof(wcex); 
	wcex.lpfnWndProc	= (WNDPROC)myWndProc; /* procedure to call with events */
	wcex.lpszClassName	= WindowClass_name;
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW); /* prevents hourglass at startup */
	/* Menu bar */
	wcex.lpszMenuName	= MAKEINTRESOURCE(MYMENU_BAR);
	/* Custom icon */
	wcex.hIcon			= LoadIcon(hInst, MAKEINTRESOURCE(MYMAIN_BIG_ICON));
	wcex.hIconSm		= LoadIcon(hInst, MAKEINTRESOURCE(MYMAIN_SM_ICON));
	RegisterClassEx(&wcex);

	// Create a window of that class
	hWnd = CreateWindow(WindowClass_name, 
			"Stupid Default Window Name", 
			WS_OVERLAPPEDWINDOW,
			CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInst, NULL);
	ShowWindow(hWnd,nCmdShow); // <- windows start invisible, so must show them...


	// Attach an editor subwindow
	hEdit1 = CreateWindow("EDIT",
			"Stupid Default Window Text\nWith Newlines...", 
			WS_CHILD | WS_BORDER | WS_HSCROLL| WS_VSCROLL |
				ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_MULTILINE ,
			0,0, // (x,y) initial position of top left corner
			200, 100, // (w,h) initial size of window
			hWnd, // parent window
			(HMENU)14, // ID to pass up to parent window
			hInst, 
			NULL); // lParam for WM_CREATE message
	ShowWindow(hEdit1,nCmdShow); // <- windows start invisible, so must show them...

	// Send a message to select some text.  This selects characters 10 through 15.
	SendMessage(hEdit1, EM_SETSEL, (WPARAM)10, (LPARAM)15);


	// Attach an editor subwindow
	hBut1 = CreateWindow("BUTTON",
			"Click Me! ", 
			WS_CHILD ,
			300,50, // (x,y) initial position of top left corner
			100, 50, // (w,h) initial size of window
			hWnd, // parent window
			(HMENU)15, // ID to pass up to parent window
			hInst, 
			NULL); // lParam for WM_CREATE message
	ShowWindow(hBut1,nCmdShow); // <- windows start invisible, so must show them...


	// Handle events
	MSG msg;
	HACCEL hAccelTable = LoadAccelerators(hInst, MAKEINTRESOURCE(MYMENU_BAR));
	while (1==GetMessage(&msg, hWnd, 0, 0)) {
			if (0!=TranslateAccelerator(hWnd, hAccelTable, &msg)) continue;
			TranslateMessage(&msg);
			DispatchMessage(&msg); /* calls myWndProc, which calls classWndProc */
	}
	return 0;
}

// "Window Procedure": the main place events get handled
LRESULT CALLBACK myWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
/*Keyboard*/
	case WM_CHAR: case WM_SYSCHAR:
		print()<<"Key typed: id="<<LOWORD(wParam)<<"\n";
		break;
	case WM_KEYDOWN: case WM_KEYUP:
	case WM_SYSKEYDOWN: case WM_SYSKEYUP:
		print()<<"Key pressed: id="<<LOWORD(wParam)<<"\n";
		break;
/*Mouse*/
	case WM_LBUTTONDOWN: case WM_LBUTTONUP:
	case WM_MBUTTONDOWN: case WM_MBUTTONUP:
	case WM_RBUTTONDOWN: case WM_RBUTTONUP:
	case WM_MOUSEMOVE:
		handleMouse(LOWORD(lParam),HIWORD(lParam),(int)wParam,message);
		return 0;
/*Menu bar / dialog item */
	case WM_COMMAND:
		print()<<"WM_COMMAND: wParam="<<HIWORD(wParam)<<"/"<<LOWORD(wParam)<<"\n";
		switch (LOWORD(wParam)) { /* decide which menu item was selected... */
		case MYMENU_EXIT: exit(0); break;
		case MYMENU_OPEN: 
			DialogBox(NULL, MAKEINTRESOURCE(MYDIALOG_ABOUT),
					hWnd, (DLGPROC)myAboutBox);
			break;
		case 14: print()<<"Edit box is doin' somethin'\n"; break;
		case 15: print()<<"The button is pressed!  Flee!\n"; break;
		default: 
			MessageBox(hWnd,"Er, I don't know what to do for that menu item.  Sorry...","Crap!",0);
			break;
		};
		return 0;
/*Window*/
	case WM_WINDOWPOSCHANGED: {
		int x=0, y=200;
		RECT rc;
		GetClientRect(hWnd, &rc);
		//print()<<"WM_WINDOWPOSCHANGED to "<<rc.left<<","<<rc.top<<"  "<<rc.right<<","<<rc.bottom<<"\n";
		MoveWindow(hEdit1,x,y,rc.right-x,rc.bottom-y,true);
		} return 1;
	case WM_ERASEBKGND:
		/*Just ignore this, which prevents flashing on redraws*/
		return 1;
	case WM_PAINT: /* have to draw stuff inside window from scratch */
		windowRepaint();
		return 0;
	case WM_CLOSE: //Asked to close window
		print()<<"WM_CLOSE\n";
		exit(0); /* Just exit normally */
		return 0;
	default:
		//WT1("                      (unknown 0x%04x)",message)
		break;
	}
	/* Pass everything not handled yet on to the default window handling */
	return DefWindowProc(hWnd, message, wParam, lParam);
}

/* Called to draw window from scratch */
void windowRepaint(void) 
{
	/* Every redraw has to be wrapped in BeginPaint/EndPaint, or
	else Windows doesn't think it actually happened... */
	PAINTSTRUCT ps;
	HDC hdc=BeginPaint(hWnd, &ps);

	/* Clear the interior of the window */
	RECT rt;
	GetClientRect(hWnd, &rt);
	FillRect(hdc, &rt, (HBRUSH)(COLOR_WINDOW+1));

	/* Draw some text in the window */
	std::string str="";
	for (int i=0;i<nBuf;i++) str+=printBuf[(curBuf+i)%nBuf].str();
	DrawText(hdc, &str[0], str.size(), &rt, DT_LEFT);

	EndPaint(hWnd, &ps);
}


/* The mouse was moved or pressed */
void handleMouse(int x,int y,int keys,UINT msg) 
{
	int buttonMask=0;
	if (keys & MK_LBUTTON) buttonMask|=(1<<0);
	if (keys & MK_MBUTTON) buttonMask|=(1<<1);
	if (keys & MK_RBUTTON) buttonMask|=(1<<2);

}

// Mesage handler for about box.
LRESULT CALLBACK myAboutBox(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			return TRUE;
		}
		break;
	}
    return FALSE;
}
